local Me=CreateFrame("Frame","vZifnabsLocator")
vZifnabs.mod.locator=Me
local mod=vZifnabs.mod
local db=nil

local nr_of_zone={}
local zone_nr_next=1
local widget={}
local ticktack=0

local FRACT=1000
local POS_MULTI=100*FRACT
local ZONE_MULTI=POS_MULTI*POS_MULTI

-- -----------------------------------------------------------------
-- INITIALIZE
-- -----------------------------------------------------------------
function Me:Initialize()
	db=vZifnabsDB
	mod.core:RegisterEvent("TICK","locator",Me.Tick)
	if not db.locator then db.locator={waypoint={}} end
	for nr,name in pairs(db.locator.zones) do
		nr_of_zone[name]=nr
		zone_nr_next=math.max(zone_nr_next,nr+1)
	end
	widget.postext=getglobal("vZifnabs_LocatorPos_Text")
	widget.postext:SetTextColor(1,1,1,1)

	widget.mm=getglobal("vZifnabs_Locator_MM")
	widget.mm:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeFile = nil});
	widget.mm:SetBackdropColor(0,0,1,1);
end

-- -----------------------------------------------------------------
-- BUTTON
-- -----------------------------------------------------------------
function Me:Button(button,fnk)
	local x,y,zone,pos=Me:GetLoc()
	if fnk=="MM" and db.locator.waypoint[UnitName("player")] then	
		db.locator.waypoint[UnitName("player")]=nil
	elseif fnk=="MM" then
		Me:SetMM(zone,x,y,date("%d.%m.%y, %H:%M:%S",time()))
	end
end

-- -----------------------------------------------------------------
-- SET MM
-- -----------------------------------------------------------------
function Me:SetMM(zone,x,y,comment)
	mod.core:Print(">> "..zone.." ("..x.."/"..y..")")	
	db.locator.waypoint[UnitName("player")]={x=x,y=y,zone=zone,comment=comment}
end

-- -----------------------------------------------------------------
-- CLOCK
-- -----------------------------------------------------------------
function Me:Tick()
	local x,y,zone=Me:GetLoc()
	local waypoint=db.locator.waypoint[UnitName("player")]
	local color={red=1,green=0,blue=0}
	local point={x=0,y=-60,tt=""}
	if not waypoint then color={red=0,green=0,blue=1}
	elseif waypoint.zone~=zone then 
		color={red=1,green=1,blue=1}
		point.tt="("..waypoint.x.."/"..waypoint.y..") "..waypoint.zone
	else
		local distance=math.sqrt((x-waypoint.x)*(x-waypoint.x)+(y-waypoint.y)*(y-waypoint.y))
		local norm=60/distance
		if distance<10 then
			color={red=0,green=1,blue=0}
			norm=60/10
		elseif distance<15 then
			color={red=(distance-5)/5.0,green=1,blue=0}
		elseif distance<20 then
			color={red=1,green=(15-distance)/5.0,blue=0}
		end
		point.x=norm*(waypoint.x-x)
		point.y=norm*(y-waypoint.y)
		point.tt="("..waypoint.x.."/"..waypoint.y..") >>"..math.floor(10*distance)/10
		color.blue=ticktack
		ticktack=1-ticktack
	end
	if (waypoint and waypoint.comment) then point.tt=point.tt.."\n("..waypoint.comment..")" end
	widget.mm:SetPoint("CENTER","Minimap","CENTER",point.x,point.y)
	widget.mm:SetBackdropColor(color.red,color.green,color.blue,1)
	widget.mm.tooltipText=point.tt
	
	x=math.floor((100*x+5)/10)/10
	y=math.floor((100*y+5)/10)/10
	if not string.find(" "..x,"[.]") then x=x..".0" end
	if not string.find(" "..y,"[.]") then y=y..".0" end
	widget.postext:SetText(x.." / "..y)	
end

-- -----------------------------------------------------------------
-- GET LOCATION
-- -----------------------------------------------------------------
function Me:GetLoc()
	SetMapToCurrentZone()
	local zone=GetZoneText()
	local zone_nr=Me:GetZoneNr(zone)
	local x, y = GetPlayerMapPosition("player")
	x=math.min(POS_MULTI-1,math.floor(POS_MULTI*x+0.5))
	y=math.min(POS_MULTI-1,math.floor(POS_MULTI*y+0.5))
	return x/FRACT,y/FRACT,zone,ZONE_MULTI*zone_nr+POS_MULTI*x+y
end

-- -----------------------------------------------------------------
-- PARSE LOCATION NUMBER
-- -----------------------------------------------------------------
function Me:ParseLoc(loc)
	local zone_nr=math.floor(loc/ZONE_MULTI)
	loc=loc-zone_nr*ZONE_MULTI
	local x=math.floor(loc/POS_MULTI)
	local y=loc-x*POS_MULTI
	return x/FRACT,y/FRACT,db.locator.zones[zone_nr]
end

-- -----------------------------------------------------------------
-- GET ZONE NUMBER
-- -----------------------------------------------------------------
function Me:GetZoneNr(zone)
	local zone_nr=nr_of_zone[zone]
	if not zone_nr then
		zone_nr=zone_nr_next
		db.locator.zones[zone_nr]=zone
		nr_of_zone[zone]=zone_nr
		zone_nr_next=zone_nr_next+1
	end
	return zone_nr
end