local vZifnabsRelease="r180501a" 

local Me=CreateFrame("Frame","vZifnabsCore")
vZifnabs={mod={core=Me},release=vZifnabsRelease}

-- Database default definitions
vZifnabsDB={frames={}}

-- Slash Commands
SLASH_VZIFNABS1 = "/z"
SlashCmdList["VZIFNABS"]=function (cmd) Me:SlashCommand(cmd) end

-- Local shortcuts to globals
local L={}

-- local work variables
local buttonlist={}
local eventlist={}
local lasttime=0
local tick={time=0,last=0,robin=0}
local updates=0
local fps={}


-- -----------------------------------------
-- INITIALIZE
-- -----------------------------------------
function Me:Initialize()
	L=vZifnabs.L
	if not vZifnabsDB.core then vZifnabsDB.core={FPS={min=20,max=40}} end
	fps=vZifnabsDB.core.FPS

	for frame,pos in pairs(vZifnabsDB.frames) do
		getglobal(frame):SetPoint(pos.point,"UIParent",pos.point,pos.x,pos.y)
	end

	for name,mod in pairs(vZifnabs.mod) do
		if name~="core" and mod.Initialize then mod:Initialize() end
	end
	Me:Print(Me:Repl(L.msg.initialized,"<release>",vZifnabsRelease))
end

-- -----------------------------------------
-- REGISTER EVENT
-- -----------------------------------------
function Me:RegisterEvent(event,mod,method)
	if not eventlist[event] then 
		eventlist[event]={} 
		getglobal("vZifnabs_EventFrame"):RegisterEvent(event)
	end
	if event=="TICK" then 
		eventlist.TICK[1+getn(eventlist.TICK)]={mod=mod,method=method,active=true}
	else	
		eventlist[event][mod]={method=method,active=true}
	end
end

-- -----------------------------------------
-- ON EVENT
-- -----------------------------------------
function Me:OnEvent(event,arg1,arg2,arg3,arg4)
	if event=="VARIABLES_LOADED" then Me:Initialize()
	elseif eventlist[event] then
		for mod,info in pairs(eventlist[event]) do
			if info.active then info.method(nil,event,arg1,arg2,arg3,arg4) end -- "nil" because method call via mod.method instead of mod:method
		end
	else Me:Print(Me:Repl(L.msg.unknownevent,"<event>",event))
	end
end

-- -----------------------------------------
-- ON UPDATE
-- -----------------------------------------
function Me:OnUpdate(elapsed)
	local newtime=time()
	updates=updates+1
	tick.time=tick.time+elapsed
	if tick.time>tick.last+0.1 or tick.robin>0 then
		if tick.robin==0 then tick.last=tick.time end
		tick.robin=tick.robin+1
		if tick.robin> getn(eventlist.TICK) then 
			tick.robin=0
		else 
			eventlist.TICK[tick.robin].method(nil,"TICK",tick.time)
		end
	end
	if newtime>lasttime then
		-- Show Clock
		getglobal("vZifnabs_Control_Clock"):SetText(date("%H:%M:%S",newtime))
		
		-- Show FPS
		fps.value=math.floor(updates/(newtime-lasttime))
		local fpstext=getglobal("vZifnabs_Control_FPS")
		local colorrange=(fps.max-fps.min)/2
		local red=min(colorrange,max(0,fps.max-fps.value))/colorrange
		local green=min(colorrange,max(0,fps.value-fps.min))/colorrange
		fpstext:SetText(fps.value)
		fpstext:SetTextColor(red,green,0,1)
		
		Me:OnEvent("CLOCK",newtime)
		
		-- Reset counter
		updates=0
		lasttime=newtime
	end
	Me:OnEvent("ONUPDATE")
end

-- -----------------------------------------
-- SLASH COMMAND
-- -----------------------------------------
function Me:SlashCommand(cmd)
	local word=Me:Split(string.lower(cmd)," ")
	if word[1]=="debug" then
		if not word[2] or word[2]=="on" then db.debug=true
		elseif word[2]=="off" then db.debug=false
		else Me:Print(Me:Repl(L.msg.undefparm,"<parm>",word[2] or "nil"))
		end
	elseif word[1]=="remind" then
		word=Me:Split(cmd," ",3)
		vZifnabs.mod.scheduler:Remind(tonumber(word[2]),word[3])
	elseif word[1]=="buffs" then vZifnabs.mod.bufmon:List()
	else
		vZifnabs.mod.emotor:SlashCommand(cmd)
	end
end

-- -----------------------------------------
-- FRAME POSITION
-- -----------------------------------------
function Me:FramePosition(frame)
	local point,relativeTo,relativePoint,xOffset,yOffset=frame:GetPoint(1)
	vZifnabsDB.frames[frame:GetName()]={point=point,x=xOffset,y=yOffset}
end

-- -----------------------------------------
-- TOOLTIP
-- -----------------------------------------
function Me:ToolTip(widget)
	if widget.tooltipText then
		GameTooltip:SetOwner(widget, "ANCHOR_RIGHT")
		GameTooltip:SetText(widget.tooltipText, 1.0, 0.8, 0.0, 1 )
		GameTooltip:Show();
	end
end
--<OnEnter>vZifnabs:ToolTip(self)</OnEnter><OnLeave>GameTooltip:Hide()</OnLeave>

-- -----------------------------------------
-- REGISTER BUTTON
-- -----------------------------------------
function Me:RegisterButton(mod,fnk,def,name)
	if not name then 
		buttonlist.ctlnr=1+(buttonlist.ctlnr or 0)
		name="vZifnabs_Control_"..buttonlist.ctlnr
	end
	local button=getglobal(name)
	if button==nil then Me:Print("ERROR: unexistant button: '"..name.."'")
	else
		buttonlist[name]={mod=mod,fnk=fnk}
		button:SetText(def.text or "?")
		button:Show()
	end
	if def.tt then 
		button.tooltipText=def.tt 		
	end
end

-- -----------------------------------------
-- BUTTON
-- -----------------------------------------
function Me:Button(button,mod,fnk)
	if not mod then
		mod=buttonlist[button:GetName()].mod
		fnk=buttonlist[button:GetName()].fnk
	end
	vZifnabs.mod[mod]:Button(button,fnk)
end

-- -----------------------------------------
-- PRINT
-- -----------------------------------------
function Me:Print(text,soundname,noprefix)
	if not noprefix then text="|cffddbb00vZifnabs:|r "..text end
	DEFAULT_CHAT_FRAME:AddMessage(text)
	if soundname~=nil then PlaySound(soundname) end
end

-- -----------------------------------------
-- PRINT
-- -----------------------------------------
function Me:Color(text,color,ambient)
	return "|c"..color..(ambient or "00")..text.."|r"
end

-- -----------------------------------------
-- REPL
-- -----------------------------------------
function Me:Repl(text,search,repl)
	local result=text
	if type(search)~="table" then result=Me:gsub_replacement(result,search,repl)
	else
		for n=1,getn(search),1 do
			result=Me:gsub_replacement(result,search[n],repl[n])
		end
	end
	return result
end

function Me:gsub_replacement(text,search,replacement)
	local searchlen=strlen(search)
	local i=0
	while text and i<string.len(text) do
		if string.sub(text,i,i+searchlen-1)~=search then i=i+1
		else
			text=string.sub(text,1,i-1)..replacement..string.sub(text,i+searchlen)
			i=i+strlen(replacement)
		end
	end
	return text
end


-- -----------------------------------------
-- TRIM
-- -----------------------------------------
function Me:Trim(text)
	while text and string.len(text)>0 and string.sub(text,1,1)==" " do 
		text=string.sub(text,2) 
	end
	while text and string.len(text)>0 and string.sub(text,-1,-1)==" " do 
		text=string.sub(text,1,-2) 
	end
	return text
end

-- -----------------------------------------
-- SPLIT
-- -----------------------------------------
function Me:Split(text,separator,maxparts)
	local part={}
	local pn=0
	while text and text~="" and (not maxparts or pn<maxparts) do
		if separator==" " then text=Me:Trim(text) end
		local ix=string.find(text..separator,separator)
		pn=pn+1
		if maxparts and pn==maxparts then part[pn]=text
		else
			part[pn]=string.sub(text,1,ix-1)
			text=string.sub(text,ix+string.len(separator))
		end
	end
	return part
end
